#version 330
#extension GL_ARB_separate_shader_objects : require
void CalculateDiffuseLighting( const in vec3 i_normal_world,
    const in vec3 i_light_direction, const in vec3 i_light_color,
    out vec3 o_lighting_diffuse )
{

    float lambertTerm;
    {
        float dotProduct;
        {

            vec3 lightVector = -i_light_direction;
            dotProduct = dot( i_normal_world, lightVector );
        }




        lambertTerm = max( 0.0, dotProduct );
    }

    o_lighting_diffuse = lambertTerm * i_light_color;
}
void CalculateSpecularLighting( const in vec3 i_position_world, const in vec3 i_normal_world, const in vec3 i_camera_position,
    const in vec3 i_light_direction, const in vec3 i_light_color,
    const in float i_shininess,
    out vec3 o_lighting_specular )
{
    float specularAmount;
    {

        float dotProduct;
        {

            vec3 reflectedDirection = reflect( i_light_direction, i_normal_world );

            vec3 viewDirection = normalize( i_camera_position - i_position_world );

            dotProduct = dot( reflectedDirection, viewDirection );

            dotProduct = max( 0.0, dotProduct );
        }


        specularAmount = pow( dotProduct, i_shininess );
    }

    o_lighting_specular = i_light_color * specularAmount;
}


uniform vec3 g_MatColor;
uniform float g_Alpha;
uniform vec3 g_LightDirection;
uniform vec3 g_LightColor;
uniform vec3 g_Ambient;
uniform float g_Shininess;
uniform vec3 g_CameraPosition;
uniform sampler2D g_TexSampler;
layout( location = 0 ) in vec4 i_color;
layout( location = 1 ) in vec2 i_texcoord;
layout( location = 2 ) in vec3 i_normal;
layout( location = 3 ) in vec3 i_position;






out vec4 o_color;

void main()

{


	{
		o_color = i_color;
		o_color *= texture2D( g_TexSampler, i_texcoord );
		o_color.rgb *= g_MatColor;
		o_color.a *= g_Alpha;
	}

	{


		vec3 light_direction = normalize( g_LightDirection );
		vec3 lighting_diffuse;
		{
			CalculateDiffuseLighting( i_normal,
				light_direction, g_LightColor,
				lighting_diffuse );
		}
		vec3 lighting_specular;
		{
			CalculateSpecularLighting( i_position, i_normal, g_CameraPosition,
				light_direction, g_LightColor,
				g_Shininess,
				lighting_specular );
		}

		o_color.rgb *= ( lighting_diffuse + g_Ambient );

		o_color.rgb += lighting_specular;
	}
}
